@@ -278,6 +278,8 @@ namespace OrthancPlugins
         Orthanc::ResourceType level = Orthanc::StringToResourceType(levels[i].c_str());
 
         const Json::Value& content = configuration[EXTRA_MAIN_DICOM_TAGS][levels[i]];
+        std::set<Orthanc::DicomTag> defaultTags;
+        Orthanc::DicomMap::GetMainDicomTags(defaultTags, level);
 
         if (content.size() > 0)
         {
@@ -286,7 +288,10 @@ namespace OrthancPlugins
             const std::string& tagName = content[t].asString();
             Orthanc::DicomTag tag(0, 0);
             OrthancPlugins::ParseTag(tag, tagName);
-            Orthanc::DicomMap::AddMainDicomTag(tag, level);
+            if (defaultTags.find(tag) == defaultTags.end()) // don't generate an error if the ExtraMainDicomTags is now a default one
+            {
+              Orthanc::DicomMap::AddMainDicomTag(tag, level);
+            }
           }
         }
       }
@@ -470,11 +475,10 @@ namespace OrthancPlugins
 
 
     static bool LookupHttpHeader2(std::string& value,
-                                  const HttpClient::HttpHeaders& headers,
+                                  const HttpHeaders& headers,
                                   const std::string& name)
     {
-      for (HttpClient::HttpHeaders::const_iterator
-             it = headers.begin(); it != headers.end(); ++it)
+      for (HttpHeaders::const_iterator it = headers.begin(); it != headers.end(); ++it)
       {
         if (boost::iequals(it->first, name))
         {
@@ -487,7 +491,7 @@ namespace OrthancPlugins
     }
 
 
-    std::string GetBasePublicUrl(const HttpClient::HttpHeaders& headers)
+    std::string GetBasePublicUrl(const HttpHeaders& headers)
     {
       assert(dicomWebConfiguration_.get() != NULL);
       std::string host = dicomWebConfiguration_->GetStringValue("Host", "");
@@ -558,7 +562,7 @@ namespace OrthancPlugins
 
     std::string GetBasePublicUrl(const OrthancPluginHttpRequest* request)
     {
-      HttpClient::HttpHeaders headers;
+      HttpHeaders headers;
 
       std::string value;
       if (LookupHttpHeader(value, request, "forwarded"))
@@ -682,6 +686,10 @@ namespace OrthancPlugins
       return GetBooleanValue("EnableMetadataCache", true);
     }
 
+    bool IsReadOnly()
+    {
+      return globalConfiguration_->GetBooleanValue("ReadOnly", false);
+    }
     
     MetadataMode GetMetadataMode(Orthanc::ResourceType level)
     {
