@@ -501,26 +501,8 @@ static void CheckStowAnswer(const Json::Value& response,
 }
 
 
-// static void AddResourceForJobContent(Json::Value resourcesForJobContent /* out */, const char* resourceType, const std::string& resourceId)
 static void AddResourceForJobContent(Json::Value& resourcesForJobContent /* out */, Orthanc::ResourceType resourceType, const std::string& resourceId)
 {
-  // const char* resourceGroup = "Instances";
-  // if (resourceType == "Study")
-  // {
-  //   resourceGroup = "Studies";
-  // }
-  // else if (resourceType == "Series")
-  // {
-  //   resourceGroup = "Series";
-  // }
-  // else if (resourceType == "Patient")
-  // {
-  //   resourceGroup = "Patients";
-  // }
-  // else if (resourceType == "Instance")
-  // {
-  //   resourceGroup = "Instances";
-  // }
   const char* resourceGroup = Orthanc::GetResourceTypeText(resourceType, true, true);
 
   if (!resourcesForJobContent.isMember(resourceGroup))
@@ -616,16 +598,16 @@ private:
     Action_Cancel
   };
   
-  boost::mutex                             mutex_;
-  std::string                              serverName_;
-  std::vector<std::string>                 instances_;
-  OrthancPlugins::HttpClient::HttpHeaders  headers_;
-  std::string                              boundary_;
-  size_t                                   position_;
-  Action                                   action_;
-  size_t                                   networkSize_;
-  bool                                     debug_;
-  Json::Value                              resourcesForJobContent_;
+  boost::mutex                 mutex_;
+  std::string                  serverName_;
+  std::vector<std::string>     instances_;
+  OrthancPlugins::HttpHeaders  headers_;
+  std::string                  boundary_;
+  size_t                       position_;
+  Action                       action_;
+  size_t                       networkSize_;
+  bool                         debug_;
+  Json::Value                  resourcesForJobContent_;
 
   bool ReadNextInstance(std::string& dicom,
                         JobContext& context)
@@ -745,6 +727,7 @@ private:
         boost::mutex::scoped_lock lock(that_.mutex_);
         context.SetContent("InstancesCount", boost::lexical_cast<std::string>(that_.instances_.size()));
         context.SetContent("Resources", that_.GetResourcesForJobContent());
+        context.SetContent("Server", that_.GetServerName());
         serverName = that_.serverName_;
         
         startPosition = that_.position_;        
@@ -757,7 +740,7 @@ private:
         client->AddHeaders(that_.headers_);
       }
 
-      OrthancPlugins::HttpClient::HttpHeaders answerHeaders;
+      OrthancPlugins::HttpHeaders answerHeaders;
       Json::Value answerBody;
 
       assert(client.get() != NULL);
@@ -839,7 +822,7 @@ private:
 public:
   StowClientJob(const std::string& serverName,
                 const std::list<std::string>& instances,
-                const OrthancPlugins::HttpClient::HttpHeaders& headers,
+                const OrthancPlugins::HttpHeaders& headers,
                 const Json::Value& resourcesForJobContent) :
     SingleFunctionJob("DicomWebStowClient"),
     serverName_(serverName),
@@ -889,6 +872,11 @@ public:
   {
     return resourcesForJobContent_;
   }
+
+  const std::string& GetServerName()
+  {
+    return serverName_;
+  }
 };
 
 
