@@ -102,4 +102,8 @@ void RetrieveStudyRendered(OrthancPluginRestOutput* output,
                            const char* url,
                            const OrthancPluginHttpRequest* request);
 
-void SetPluginCanDownloadTranscodedFile(bool enable);
\ No newline at end of file
+void SetPluginCanDownloadTranscodedFile(bool enable);
+
+void SetPluginCanUseExtendedFile(bool enable);
+
+void SetSystemIsReadOnly(bool isReadOnly);
\ No newline at end of file
